@@ -25,8 +25,8 @@ if (mat) { ncol <- length(answer[[1]])  #the more complicated case. How to reord
 	row.names <-attr(answer[[1]],"row.names")
 	dim.names <- attr(answer,"dimnames")
   
-	 mat.ans <- matrix(NA,ncol=ncol,nrow=n.var*n.groups)
-	 labels.ans <- matrix(NA,ncol = n.grouping+1,nrow= n.var*n.groups)
+	 mat.ans <- matrix(NaN,ncol=ncol,nrow=n.var*n.groups)
+	 labels.ans <- matrix(NaN,ncol = n.grouping+1,nrow= n.var*n.groups)
 	 colnames(labels.ans) <- c("item",paste("group",1:n.grouping,sep=""))
 	 colnames(mat.ans) <- colnames(answer[[1]])
 	 rn <- 1:(n.var*n.groups)
